@@ -61,6 +61,9 @@ struct _GESClipPrivate
 
   guint nb_effects;
 
+  GList *copied_track_elements;
+  GESLayer *copied_layer;
+
   /* The formats supported by this Clip */
   GESTrackType supportedformats;
 };
@@ -352,6 +355,8 @@ _child_added (GESContainer * container, GESTimelineElement * element)
 {
   g_signal_connect (G_OBJECT (element), "notify::priority",
       G_CALLBACK (_child_priority_changed_cb), container);
+
+  _child_priority_changed_cb (element, NULL, container);
   _compute_height (container);
 }
 
@@ -625,21 +630,43 @@ _edit (GESContainer * container, GList * layers,
   return ret;
 }
 
-static gboolean
+static void
+_deep_copy (GESTimelineElement * element, GESTimelineElement * copy)
+{
+  GList *tmp;
+  GESClip *self = GES_CLIP (element), *ccopy = GES_CLIP (copy);
+
+  for (tmp = GES_CONTAINER_CHILDREN (element); tmp; tmp = tmp->next) {
+    ccopy->priv->copied_track_elements =
+        g_list_append (ccopy->priv->copied_track_elements,
+        ges_timeline_element_copy (tmp->data, TRUE));
+  }
+
+  if (self->priv->copied_layer)
+    ccopy->priv->copied_layer = g_object_ref (self->priv->copied_layer);
+  else if (self->priv->layer)
+    ccopy->priv->copied_layer = g_object_ref (self->priv->layer);
+}
+
+static GESTimelineElement *
 _paste (GESTimelineElement * element, GESTimelineElement * ref,
     GstClockTime paste_position)
 {
   GList *tmp;
   GESClip *self = GES_CLIP (element);
-  GESClip *refclip = GES_CLIP (ref);
+  GESClip *nclip = GES_CLIP (ges_timeline_element_copy (element, FALSE));
 
-  ges_clip_set_moving_from_layer (self, TRUE);
-  ges_layer_add_clip (refclip->priv->layer, self);
-  ges_clip_set_moving_from_layer (self, FALSE);
+  if (self->priv->copied_layer)
+    nclip->priv->copied_layer = g_object_ref (self->priv->copied_layer);
 
-  ges_timeline_element_set_start (GES_TIMELINE_ELEMENT (self), paste_position);
+  ges_clip_set_moving_from_layer (nclip, TRUE);
+  if (self->priv->copied_layer)
+    ges_layer_add_clip (self->priv->copied_layer, nclip);
+  ges_clip_set_moving_from_layer (nclip, FALSE);
 
-  for (tmp = GES_CONTAINER_CHILDREN (refclip); tmp; tmp = tmp->next) {
+  ges_timeline_element_set_start (GES_TIMELINE_ELEMENT (nclip), paste_position);
+
+  for (tmp = self->priv->copied_track_elements; tmp; tmp = tmp->next) {
     GESTrackElement *new_trackelement, *trackelement =
         GES_TRACK_ELEMENT (tmp->data);
 
@@ -651,7 +678,7 @@ _paste (GESTimelineElement * element, GESTimelineElement * ref,
       continue;
     }
 
-    ges_container_add (GES_CONTAINER (self),
+    ges_container_add (GES_CONTAINER (nclip),
         GES_TIMELINE_ELEMENT (new_trackelement));
 
     ges_track_element_copy_properties (GES_TIMELINE_ELEMENT (trackelement),
@@ -661,7 +688,7 @@ _paste (GESTimelineElement * element, GESTimelineElement * ref,
         GST_CLOCK_TIME_NONE);
   }
 
-  return TRUE;
+  return GES_TIMELINE_ELEMENT (nclip);
 }
 
 
@@ -703,6 +730,16 @@ ges_clip_set_property (GObject * object, guint property_id,
   }
 }
 
+static void
+ges_clip_finalize (GObject * object)
+{
+  GESClip *self = GES_CLIP (object);
+
+  g_list_free_full (self->priv->copied_track_elements, g_object_unref);
+
+  G_OBJECT_CLASS (ges_clip_parent_class)->finalize (object);
+}
+
 static void
 ges_clip_class_init (GESClipClass * klass)
 {
@@ -714,6 +751,7 @@ ges_clip_class_init (GESClipClass * klass)
 
   object_class->get_property = ges_clip_get_property;
   object_class->set_property = ges_clip_set_property;
+  object_class->finalize = ges_clip_finalize;
   klass->create_track_elements = ges_clip_create_track_elements_func;
   klass->create_track_element = NULL;
 
@@ -754,6 +792,7 @@ ges_clip_class_init (GESClipClass * klass)
   element_class->set_priority = _set_priority;
   element_class->set_max_duration = _set_max_duration;
   element_class->paste = _paste;
+  element_class->deep_copy = _deep_copy;
 
   container_class->add_child = _add_child;
   container_class->remove_child = _remove_child;
@@ -785,8 +824,8 @@ ges_clip_init (GESClip * self)
  * keep a reference to the newly created trackelement, you therefore need to
  * call @ges_container_remove when you are done with it.
  *
- * Returns: (transfer none): A #GESTrackElement. Returns NULL if the #GESTrackElement could not
- * be created.
+ * Returns: (transfer none) (nullable): A #GESTrackElement. Returns NULL if
+ * the #GESTrackElement could not be created.
  */
 GESTrackElement *
 ges_clip_create_track_element (GESClip * clip, GESTrackType type)
@@ -830,7 +869,7 @@ ges_clip_create_track_element (GESClip * clip, GESTrackType type)
 GList *
 ges_clip_create_track_elements (GESClip * clip, GESTrackType type)
 {
-  GList *result, *tmp;
+  GList *result = NULL, *tmp, *children;
   GESClipClass *klass;
   guint max_prio, min_prio;
 
@@ -845,7 +884,23 @@ ges_clip_create_track_elements (GESClip * clip, GESTrackType type)
 
   GST_DEBUG_OBJECT (clip, "Creating TrackElements for type: %s",
       ges_track_type_name (type));
-  result = klass->create_track_elements (clip, type);
+  children = ges_container_get_children (GES_CONTAINER (clip), TRUE);
+  for (tmp = children; tmp; tmp = tmp->next) {
+    GESTrackElement *child = GES_TRACK_ELEMENT (tmp->data);
+
+    if (!GES_IS_BASE_EFFECT (child) && !ges_track_element_get_track (child) &&
+        ges_track_element_get_track_type (child) & type) {
+
+      GST_DEBUG_OBJECT (clip, "Removing for reusage: %" GST_PTR_FORMAT, child);
+      result = g_list_prepend (result, g_object_ref (child));
+      ges_container_remove (GES_CONTAINER (clip), tmp->data);
+    }
+  }
+  g_list_free_full (children, gst_object_unref);
+
+  if (!result) {
+    result = klass->create_track_elements (clip, type);
+  }
 
   _get_priority_range (GES_CONTAINER (clip), &min_prio, &max_prio);
   for (tmp = result; tmp; tmp = tmp->next) {
@@ -1014,8 +1069,8 @@ ges_clip_move_to_layer (GESClip * clip, GESLayer * layer)
  * Note: If many objects match, then the one with the highest priority will be
  * returned.
  *
- * Returns: (transfer full): The #GESTrackElement used by @track, else %NULL,
- * Unref after usage
+ * Returns: (transfer full) (nullable): The #GESTrackElement used by @track,
+ * else %NULL. Unref after usage
  */
 
 GESTrackElement *
@@ -1051,7 +1106,7 @@ ges_clip_find_track_element (GESClip * clip, GESTrack * track, GType type)
  *
  * Get the #GESLayer to which this clip belongs.
  *
- * Returns: (transfer full): The #GESLayer where this @clip is being
+ * Returns: (transfer full) (nullable): The #GESLayer where this @clip is being
  * used, or %NULL if it is not used on any layer. The caller should unref it
  * usage.
  */
@@ -1207,23 +1262,35 @@ ges_clip_set_top_effect_index (GESClip * clip, GESBaseEffect * effect,
 /**
  * ges_clip_split:
  * @clip: the #GESClip to split
- * @position: a #GstClockTime representing the position at which to split
+ * @position: a #GstClockTime representing the timeline position at which to split
  *
- * The function modifies @clip, and creates another #GESClip so
- * we have two clips at the end, splitted at the time specified by @position.
- * The newly created clip will be added to the same layer as @clip is in.
- * This implies that @clip must be in a #GESLayer for the operation to
- * be possible.
+ * The function modifies @clip, and creates another #GESClip so we have two
+ * clips at the end, splitted at the time specified by @position, as a position
+ * in the timeline (not in the clip to be split). For example, if
+ * ges_clip_split is called on a 4-second clip playing from 0:01.00 until
+ * 0:05.00, with a split position of 0:02.00, this will result in one clip of 1
+ * second and one clip of 3 seconds, not in two clips of 2 seconds.
  *
- * Returns: (transfer none): The newly created #GESClip resulting from the
- * splitting
+ * The newly created clip will be added to the same layer as @clip is in. This
+ * implies that @clip must be in a #GESLayer for the operation to be possible.
+ *
+ * This method supports clips playing at a different tempo than one second per
+ * second. For example, splitting a clip with a #GESEffect 'pitch tempo=1.5'
+ * four seconds after it starts, will set the inpoint of the new clip to six
+ * seconds after that of the clip to split. For this, the rate-changing
+ * property must be registered using @ges_effect_class_register_rate_property;
+ * for the 'pitch' plugin, this is already done.
+ *
+ * Returns: (transfer none) (nullable): The newly created #GESClip resulting
+ * from the splitting or %NULL if the clip can't be split.
  */
 GESClip *
 ges_clip_split (GESClip * clip, guint64 position)
 {
   GList *tmp;
   GESClip *new_object;
-  GstClockTime start, inpoint, duration;
+  GstClockTime start, inpoint, duration, old_duration, new_duration;
+  gdouble media_duration_factor;
 
   g_return_val_if_fail (GES_IS_CLIP (clip), NULL);
   g_return_val_if_fail (clip->priv->layer, NULL);
@@ -1248,11 +1315,15 @@ ges_clip_split (GESClip * clip, guint64 position)
 
   GST_DEBUG_OBJECT (new_object, "New 'splitted' clip");
   /* Set new timing properties on the Clip */
+  media_duration_factor =
+      ges_timeline_element_get_media_duration_factor (GES_TIMELINE_ELEMENT
+      (clip));
+  new_duration = duration + start - position;
+  old_duration = position - start;
   _set_start0 (GES_TIMELINE_ELEMENT (new_object), position);
   _set_inpoint0 (GES_TIMELINE_ELEMENT (new_object),
-      _INPOINT (clip) + duration - (duration + start - position));
-  _set_duration0 (GES_TIMELINE_ELEMENT (new_object),
-      duration + start - position);
+      inpoint + old_duration * media_duration_factor);
+  _set_duration0 (GES_TIMELINE_ELEMENT (new_object), new_duration);
 
   /* We do not want the timeline to create again TrackElement-s */
   ges_clip_set_moving_from_layer (new_object, TRUE);
@@ -1274,9 +1345,8 @@ ges_clip_split (GESClip * clip, guint64 position)
     /* Set 'new' track element timing propeties */
     _set_start0 (GES_TIMELINE_ELEMENT (new_trackelement), position);
     _set_inpoint0 (GES_TIMELINE_ELEMENT (new_trackelement),
-        inpoint + duration - (duration + start - position));
-    _set_duration0 (GES_TIMELINE_ELEMENT (new_trackelement),
-        duration + start - position);
+        inpoint + old_duration * media_duration_factor);
+    _set_duration0 (GES_TIMELINE_ELEMENT (new_trackelement), new_duration);
 
     ges_container_add (GES_CONTAINER (new_object),
         GES_TIMELINE_ELEMENT (new_trackelement));
@@ -1287,7 +1357,7 @@ ges_clip_split (GESClip * clip, guint64 position)
         position - start + inpoint);
   }
 
-  _set_duration0 (GES_TIMELINE_ELEMENT (clip), position - _START (clip));
+  _set_duration0 (GES_TIMELINE_ELEMENT (clip), old_duration);
 
   return new_object;
 }
