@@ -19,7 +19,7 @@
 
 /**
  * SECTION:geseffect
- * @short_description: adds an effect build from a parse-launch style 
+ * @short_description: adds an effect build from a parse-launch style
  * bin description to a stream in a GESSourceClip or a GESLayer
  */
 
@@ -55,26 +55,44 @@ enum
 static gchar *
 extractable_check_id (GType type, const gchar * id, GError ** error)
 {
-  GstElement *effect = gst_parse_bin_from_description (id, TRUE, error);
+  gchar *bin_desc, *real_id;
+  GESTrackType ttype;
 
-  if (effect == NULL)
+  bin_desc = ges_effect_assect_id_get_type_and_bindesc (id, &ttype, error);
+
+  if (bin_desc == NULL)
     return NULL;
 
-  gst_object_unref (effect);
-  return g_strdup (id);
+  if (ttype == GES_TRACK_TYPE_AUDIO)
+    real_id = g_strdup_printf ("audio %s", bin_desc);
+  else if (ttype == GES_TRACK_TYPE_VIDEO)
+    real_id = g_strdup_printf ("video %s", bin_desc);
+  else
+    g_assert_not_reached ();
+
+  return real_id;
 }
 
 static GParameter *
 extractable_get_parameters_from_id (const gchar * id, guint * n_params)
 {
-  GParameter *params = g_new0 (GParameter, 2);
+  GParameter *params = g_new0 (GParameter, 3);
+  gchar *bin_desc;
+  GESTrackType ttype;
+
+  bin_desc = ges_effect_assect_id_get_type_and_bindesc (id, &ttype, NULL);
 
   params[0].name = "bin-description";
   g_value_init (&params[0].value, G_TYPE_STRING);
-  g_value_set_string (&params[0].value, id);
+  g_value_set_string (&params[0].value, bin_desc);
 
-  *n_params = 1;
+  params[1].name = "track-type";
+  g_value_init (&params[1].value, GES_TYPE_TRACK_TYPE);
+  g_value_set_flags (&params[1].value, ttype);
 
+  *n_params = 2;
+
+  g_free (bin_desc);
   return params;
 }
 
@@ -141,6 +159,11 @@ ges_effect_class_init (GESEffectClass * klass)
 
   obj_bg_class->create_element = ges_effect_create_element;
 
+  klass->rate_properties = NULL;
+  ges_effect_class_register_rate_property (klass, "scaletempo", "rate");
+  ges_effect_class_register_rate_property (klass, "pitch", "tempo");
+  ges_effect_class_register_rate_property (klass, "pitch", "rate");
+
   /**
    * GESEffect:bin-description:
    *
@@ -188,27 +211,20 @@ ges_effect_create_element (GESTrackElement * object)
 
   GError *error = NULL;
   GESEffect *self = GES_EFFECT (object);
-  GESTrack *track = ges_track_element_get_track (object);
   const gchar *wanted_categories[] = { "Effect", NULL };
 
-  if (!track) {
-    GST_WARNING
-        ("The object %p should be in a Track for the element to be created",
-        object);
-    return NULL;
-  }
+  GESTrackType type = ges_track_element_get_track_type (object);
 
-  if (track->type == GES_TRACK_TYPE_VIDEO) {
+  if (type == GES_TRACK_TYPE_VIDEO) {
     bin_desc = g_strconcat ("videoconvert name=pre_video_convert ! ",
         self->priv->bin_description, " ! videoconvert name=post_video_convert",
         NULL);
-  } else if (track->type == GES_TRACK_TYPE_AUDIO) {
+  } else if (type == GES_TRACK_TYPE_AUDIO) {
     bin_desc =
         g_strconcat ("audioconvert ! audioresample !",
         self->priv->bin_description, NULL);
   } else {
-    GST_DEBUG ("Track type not supported");
-    return NULL;
+    g_assert_not_reached ();
   }
 
   effect = gst_parse_bin_from_description (bin_desc, TRUE, &error);
@@ -234,9 +250,14 @@ ges_effect_create_element (GESTrackElement * object)
  * ges_effect_new:
  * @bin_description: The gst-launch like bin description of the effect
  *
- * Creates a new #GESEffect from the description of the bin.
+ * Creates a new #GESEffect from the description of the bin. It should be
+ * possible to determine the type of the effect through the element
+ * 'klass' metadata of the GstElements that will be created.
+ * In that corner case, you should use:
+ * #ges_asset_request (GES_TYPE_EFFECT, "audio your ! bin ! description", NULL);
+ * and extract that asset to be in full control.
  *
- * Returns: a newly created #GESEffect, or %NULL if something went
+ * Returns: (nullable): a newly created #GESEffect, or %NULL if something went
  * wrong.
  */
 GESEffect *
@@ -254,3 +275,100 @@ ges_effect_new (const gchar * bin_description)
 
   return effect;
 }
+
+static int
+property_name_compare (gconstpointer s1, gconstpointer s2)
+{
+  return g_strcmp0 ((const gchar *) s1, (const gchar *) s2);
+}
+
+/**
+ * ges_effect_class_register_rate_property:
+ * @klass: Instance of the GESEffectClass
+ * @element_name: Name of the GstElement that changes the rate
+ * @property_name: Name of the property that changes the rate
+ *
+ * Register an element that can change the rate at which media is playing. The
+ * property type must be float or double, and must be a factor of the rate,
+ * i.e. a value of 2.0 must mean that the media plays twice as fast. For
+ * example, this is true for the properties 'rate' and 'tempo' of the element
+ * 'pitch', which is already registered by default. By registering the element,
+ * timeline duration can be correctly converted into media duration, allowing
+ * the right segment seeks to be sent to the sources.
+ *
+ * A reference to the GESEffectClass can be obtained as follows:
+ *   GES_EFFECT_CLASS (g_type_class_ref (GES_TYPE_EFFECT));
+ *
+ * Returns: whether the rate property was succesfully registered. When this
+ * method returns false, a warning is emitted with more information.
+ */
+gboolean
+ges_effect_class_register_rate_property (GESEffectClass * klass,
+    const gchar * element_name, const gchar * property_name)
+{
+  GstElementFactory *element_factory = NULL;
+  GstElement *element = NULL;
+  GParamSpec *pspec = NULL;
+  gchar *full_property_name = NULL;
+  GType param_type;
+  gboolean res = FALSE;
+
+  element_factory = gst_element_factory_find (element_name);
+  if (element_factory == NULL) {
+    GST_WARNING
+        ("Did not add rate property '%s' for element '%s': the element factory could not be found",
+        property_name, element_name);
+    goto fail;
+  }
+
+  element = gst_element_factory_create (element_factory, NULL);
+  if (element == NULL) {
+    GST_WARNING
+        ("Did not add rate property '%s' for element '%s': the element could not be constructed",
+        property_name, element_name);
+    goto fail;
+  }
+
+  pspec =
+      g_object_class_find_property (G_OBJECT_GET_CLASS (element),
+      property_name);
+  if (pspec == NULL) {
+    GST_WARNING
+        ("Did not add rate property '%s' for element '%s': the element did not have the property name specified",
+        property_name, element_name);
+    goto fail;
+  }
+
+  param_type = G_PARAM_SPEC_VALUE_TYPE (pspec);
+  if (param_type != G_TYPE_FLOAT && param_type != G_TYPE_DOUBLE) {
+    GST_WARNING
+        ("Did not add rate property '%s' for element '%s': the property is not of float or double type",
+        property_name, element_name);
+    goto fail;
+  }
+
+  full_property_name = g_strdup_printf ("%s::%s",
+      g_type_name (gst_element_factory_get_element_type (element_factory)),
+      property_name);
+
+  if (g_list_find_custom (klass->rate_properties, full_property_name,
+          property_name_compare) == NULL) {
+    klass->rate_properties =
+        g_list_append (klass->rate_properties, full_property_name);
+    GST_DEBUG ("Added rate property %s", full_property_name);
+  } else {
+    g_free (full_property_name);
+  }
+
+  res = TRUE;
+
+fail:
+  if (element_factory != NULL)
+    gst_object_unref (element_factory);
+  if (element != NULL)
+    gst_object_unref (element);
+  if (pspec != NULL)
+    g_param_spec_unref (pspec);
+
+  return res;
+}
