@@ -554,16 +554,28 @@ _child_removed (GESContainer * group, GESTimelineElement * child)
 static GList *
 _ungroup (GESContainer * group, gboolean recursive)
 {
+  GPtrArray *children_array;
+  GESTimeline *timeline;
   GList *children, *tmp, *ret = NULL;
 
+  /* We choose 16 just as an arbitrary value */
+  children_array = g_ptr_array_sized_new (16);
+  timeline = GES_TIMELINE_ELEMENT_TIMELINE (group);
+
   children = ges_container_get_children (group, FALSE);
   for (tmp = children; tmp; tmp = tmp->next) {
     GESTimelineElement *child = tmp->data;
 
     gst_object_ref (child);
     ges_container_remove (group, child);
+    g_ptr_array_add (children_array, child);
     ret = g_list_append (ret, child);
   }
+
+  if (timeline)
+    ges_timeline_emit_group_removed (timeline, (GESGroup *) group,
+        children_array);
+  g_ptr_array_free (children_array, TRUE);
   g_list_free_full (children, gst_object_unref);
 
   /* No need to remove from the timeline here, this will be done in _child_removed */
@@ -598,21 +610,21 @@ _group (GList * containers)
   return ret;
 }
 
-static gboolean
+static GESTimelineElement *
 _paste (GESTimelineElement * element, GESTimelineElement * ref,
     GstClockTime paste_position)
 {
-  if (GES_TIMELINE_ELEMENT_CLASS (parent_class)->paste (element,
-          ref, paste_position)) {
+  GESTimelineElement *ngroup =
+      GES_TIMELINE_ELEMENT_CLASS (parent_class)->paste (element, ref,
+      paste_position);
 
-    if (GES_CONTAINER_CHILDREN (element))
+  if (ngroup) {
+    if (GES_CONTAINER_CHILDREN (ngroup))
       timeline_add_group (GES_TIMELINE_ELEMENT_TIMELINE (GES_CONTAINER_CHILDREN
-              (element)->data), GES_GROUP (element));
-
-    return TRUE;
+              (ngroup)->data), GES_GROUP (element));
   }
 
-  return FALSE;
+  return ngroup;
 }
 
 
@@ -780,7 +792,7 @@ ges_group_init (GESGroup * self)
  * together, it is recommanded to use the #ges_container_group method so the
  * proper subclass is selected.
  *
- * Returns: The new empty group.
+ * Returns: (transfer floating): The new empty group.
  */
 GESGroup *
 ges_group_new (void)
