@@ -31,6 +31,8 @@
 #include "ges-asset.h"
 #include "ges-base-xml-formatter.h"
 
+G_BEGIN_DECLS
+
 GST_DEBUG_CATEGORY_EXTERN (_ges_debug);
 #define GST_CAT_DEFAULT _ges_debug
 
@@ -143,11 +145,15 @@ GESAsset*
 ges_asset_cache_lookup(GType extractable_type, const gchar * id);
 
 gboolean
-ges_asset_set_proxy (GESAsset *asset, const gchar *new_id);
+ges_asset_try_proxy (GESAsset *asset, const gchar *new_id);
 
 G_GNUC_INTERNAL gboolean
 ges_asset_request_id_update (GESAsset *asset, gchar **proposed_id,
     GError *error);
+G_GNUC_INTERNAL gchar *
+ges_effect_assect_id_get_type_and_bindesc (const char    *id,
+                                           GESTrackType  *track_type,
+                                           GError       **error);
 
 /* GESExtractable internall methods
  *
@@ -224,6 +230,7 @@ G_GNUC_INTERNAL void ges_base_xml_formatter_add_asset        (GESBaseXmlFormatte
                                                                  GType extractable_type,
                                                                  GstStructure *properties,
                                                                  const gchar *metadatas,
+                                                                 const gchar *proxy_id,
                                                                  GError **error);
 G_GNUC_INTERNAL void ges_base_xml_formatter_add_layer           (GESBaseXmlFormatter *self,
                                                                  GType extractable_type,
@@ -359,6 +366,11 @@ G_GNUC_INTERNAL GESImageSource     * ges_image_source_new      (gchar *uri);
 G_GNUC_INTERNAL GESTitleSource     * ges_title_source_new      (void);
 G_GNUC_INTERNAL GESVideoTestSource * ges_video_test_source_new (void);
 
+/****************************************************
+ *              GESTimelineElement                  *
+ ****************************************************/
+G_GNUC_INTERNAL gdouble ges_timeline_element_get_media_duration_factor(GESTimelineElement *self);
+
 /******************************
  *  GESMultiFile internal API *
  ******************************/
@@ -384,4 +396,6 @@ G_GNUC_INTERNAL gboolean ges_nle_composition_add_object (GstElement *comp, GstEl
 G_GNUC_INTERNAL gboolean ges_nle_composition_remove_object (GstElement *comp, GstElement *object);
 G_GNUC_INTERNAL gboolean ges_nle_object_commit (GstElement * nlesource, gboolean recurse);
 
+G_END_DECLS
+
 #endif /* __GES_INTERNAL_H__ */
