@@ -31,6 +31,7 @@
 #include "ges-extractable.h"
 #include "ges-meta-container.h"
 #include "ges-internal.h"
+#include "ges-effect.h"
 
 #include <string.h>
 #include <gobject/gvaluecollector.h>
@@ -123,6 +124,8 @@ _lookup_child (GESTimelineElement * self, const gchar * prop_name,
       ChildPropHandler *handler = (ChildPropHandler *) value;
       if (classename == NULL ||
           g_strcmp0 (G_OBJECT_TYPE_NAME (G_OBJECT (handler->child)),
+              classename) == 0 ||
+          g_strcmp0 (g_type_name (G_PARAM_SPEC (key)->owner_type),
               classename) == 0) {
         GST_DEBUG_OBJECT (self, "The %s property from %s has been found", name,
             classename);
@@ -247,6 +250,9 @@ ges_timeline_element_dispose (GObject * object)
 {
   GESTimelineElement *self = GES_TIMELINE_ELEMENT (object);
 
+  if (self->priv->children_props)
+    g_hash_table_unref (self->priv->children_props);
+
   g_clear_object (&self->priv->copied_from);
 }
 
@@ -547,8 +553,8 @@ had_parent:
  * Returns the parent of @self. This function increases the refcount
  * of the parent object so you should gst_object_unref() it after usage.
  *
- * Returns: (transfer full): parent of @self, this can be %NULL if @self
- *   has no parent. unref after usage.
+ * Returns: (transfer full) (nullable): parent of @self, this can be %NULL if
+ * @self has no parent. unref after usage.
  */
 GESTimelineElement *
 ges_timeline_element_get_parent (GESTimelineElement * self)
@@ -623,8 +629,8 @@ had_timeline:
  * Returns the timeline of @self. This function increases the refcount
  * of the timeline so you should gst_object_unref() it after usage.
  *
- * Returns: (transfer full): timeline of @self, this can be %NULL if @self
- *   has no timeline. unref after usage.
+ * Returns: (transfer full) (nullable): timeline of @self, this can be %NULL if
+ * @self has no timeline. unref after usage.
  */
 GESTimeline *
 ges_timeline_element_get_timeline (GESTimelineElement * self)
@@ -722,7 +728,7 @@ ges_timeline_element_set_inpoint (GESTimelineElement * self,
     return;
   }
 
-  GST_WARNING_OBJECT (self, "No set_inpoint virtual method implementation"
+  GST_DEBUG_OBJECT (self, "No set_inpoint virtual method implementation"
       " on class %s. Can not set inpoint %" GST_TIME_FORMAT,
       G_OBJECT_CLASS_NAME (klass), GST_TIME_ARGS (inpoint));
 }
@@ -1259,7 +1265,7 @@ ges_timeline_element_add_child_property (GESTimelineElement * self,
 }
 
 /**
- * ges_track_element_get_child_property_by_pspec:
+ * ges_timeline_element_get_child_property_by_pspec:
  * @self: a #GESTrackElement
  * @pspec: The #GParamSpec that specifies the property you want to get
  * @value: (out): return location for the value
@@ -1362,7 +1368,7 @@ not_found:
 
 /**
 * ges_timeline_element_get_child_property:
-* @object: The origin #GESTimelineElement
+* @self: The origin #GESTimelineElement
 * @property_name: The name of the property
 * @value: (out): return location for the property value, it will
 * be initialized if it is initialized with 0
@@ -1417,7 +1423,7 @@ not_found:
  *     proper GParamSpec in case various GstElement-s contain the same property
  *     name. If you don't do so, you will get the first element found, having
  *     this property and the and the corresponding GParamSpec.
- * @element: (out) (allow-none) (transfer full): pointer to a #GstElement that
+ * @child: (out) (allow-none) (transfer full): pointer to a #GstElement that
  *     takes the real object to set property on
  * @pspec: (out) (allow-none) (transfer full): pointer to take the #GParamSpec
  *     describing the property
@@ -1694,25 +1700,27 @@ ges_timeline_element_get_track_types (GESTimelineElement * self)
  * using ges_timeline_element_copy with recurse=TRUE set,
  * otherwise it will fail.
  *
+ * Returns: (transfer none): Paste @self copying the element
+ *
  * Since: 1.6.0
  */
-gboolean
+GESTimelineElement *
 ges_timeline_element_paste (GESTimelineElement * self,
     GstClockTime paste_position)
 {
-  gboolean res;
+  GESTimelineElement *res;
   g_return_val_if_fail (GES_IS_TIMELINE_ELEMENT (self), FALSE);
 
   if (!self->priv->copied_from) {
     GST_ERROR_OBJECT (self, "Is not being 'deeply' copied!");
 
-    return FALSE;
+    return NULL;
   }
 
   if (!GES_TIMELINE_ELEMENT_GET_CLASS (self)->paste) {
     GST_ERROR_OBJECT (self, "No paste vmethod implemented");
 
-    return FALSE;
+    return NULL;
   }
 
   res = GES_TIMELINE_ELEMENT_GET_CLASS (self)->paste (self,
@@ -1720,5 +1728,49 @@ ges_timeline_element_paste (GESTimelineElement * self,
 
   g_clear_object (&self->priv->copied_from);
 
-  return res;
+  return g_object_ref (res);
+}
+
+/* Internal */
+gdouble
+ges_timeline_element_get_media_duration_factor (GESTimelineElement * self)
+{
+  gdouble media_duration_factor;
+  GESEffectClass *class;
+  GList *props;
+
+  media_duration_factor = 1.0;
+
+  class = GES_EFFECT_CLASS (g_type_class_ref (GES_TYPE_EFFECT));
+
+  for (props = class->rate_properties; props != NULL; props = props->next) {
+    GObject *child;
+    GParamSpec *pspec;
+    if (ges_timeline_element_lookup_child (self, props->data, &child, &pspec)) {
+      if (G_PARAM_SPEC_VALUE_TYPE (pspec) == G_TYPE_FLOAT) {
+        gfloat rate_change;
+        g_object_get (child, pspec->name, &rate_change, NULL);
+        media_duration_factor *= rate_change;
+      } else if (G_PARAM_SPEC_VALUE_TYPE (pspec) == G_TYPE_DOUBLE) {
+        gdouble rate_change;
+        g_object_get (child, pspec->name, &rate_change, NULL);
+        media_duration_factor *= rate_change;
+      } else {
+        GST_WARNING_OBJECT (self,
+            "Rate property %s in child %" GST_PTR_FORMAT
+            " is of unsupported type %s", pspec->name, child,
+            G_VALUE_TYPE_NAME (pspec->value_type));
+      }
+
+      gst_object_unref (child);
+      g_param_spec_unref (pspec);
+
+      GST_DEBUG_OBJECT (self,
+          "Added rate changing property %s, set to value %lf",
+          (const char *) props->data, media_duration_factor);
+    }
+  }
+
+  g_type_class_unref (class);
+  return media_duration_factor;
 }
