@@ -42,36 +42,18 @@ struct _GESTrackElementPrivate
 {
   GESTrackType track_type;
 
-  /* These fields are only used before the nleobject is available */
-  guint64 pending_start;
-  guint64 pending_inpoint;
-  guint64 pending_duration;
-  guint32 pending_priority;
-  gboolean pending_active;
-
   GstElement *nleobject;        /* The NleObject */
   GstElement *element;          /* The element contained in the nleobject (can be NULL) */
 
   GESTrack *track;
 
-  gboolean valid;
   gboolean locked;              /* If TRUE, then moves in sync with its controlling
                                  * GESClip */
 
   GHashTable *bindings_hashtable;       /* We need this if we want to be able to serialize
                                            and deserialize keyframes */
-
-  GList *pending_bindings;
 };
 
-typedef struct
-{
-  GESTrackElement *element;
-  GstControlSource *source;
-  gchar *propname;
-  gchar *binding_type;
-} PendingBinding;
-
 enum
 {
   PROP_0,
@@ -208,9 +190,52 @@ ges_track_element_dispose (GObject * object)
 }
 
 static void
-ges_track_element_finalize (GObject * object)
+ges_track_element_constructed (GObject * gobject)
 {
-  G_OBJECT_CLASS (ges_track_element_parent_class)->finalize (object);
+  GESTrackElementClass *class;
+  GstElement *nleobject;
+  gdouble media_duration_factor;
+  gchar *tmp;
+  GESTrackElement *object = GES_TRACK_ELEMENT (gobject);
+
+  GST_DEBUG_OBJECT (object, "Creating NleObject");
+
+  class = GES_TRACK_ELEMENT_GET_CLASS (object);
+  g_assert (class->create_gnl_object);
+
+  nleobject = class->create_gnl_object (object);
+  if (G_UNLIKELY (nleobject == NULL)) {
+    GST_ERROR_OBJECT (object, "Could not create NleObject");
+
+    return;
+  }
+
+  tmp = g_strdup_printf ("%s:%s", G_OBJECT_TYPE_NAME (object),
+      GST_OBJECT_NAME (nleobject));
+  gst_object_set_name (GST_OBJECT (nleobject), tmp);
+  g_free (tmp);
+
+  GST_DEBUG_OBJECT (object, "Got a valid NleObject, now filling it in");
+
+  object->priv->nleobject = gst_object_ref (nleobject);
+  g_object_set_qdata (G_OBJECT (nleobject), NLE_OBJECT_TRACK_ELEMENT_QUARK,
+      object);
+
+  /* Set some properties on the NleObject */
+  g_object_set (object->priv->nleobject,
+      "start", GES_TIMELINE_ELEMENT_START (object),
+      "inpoint", GES_TIMELINE_ELEMENT_INPOINT (object),
+      "duration", GES_TIMELINE_ELEMENT_DURATION (object),
+      "priority", GES_TIMELINE_ELEMENT_PRIORITY (object),
+      "active", object->active, NULL);
+
+  media_duration_factor =
+      ges_timeline_element_get_media_duration_factor (GES_TIMELINE_ELEMENT
+      (object));
+  g_object_set (object->priv->nleobject,
+      "media-duration-factor", media_duration_factor, NULL);
+
+  G_OBJECT_CLASS (ges_track_element_parent_class)->constructed (gobject);
 }
 
 static void
@@ -224,7 +249,7 @@ ges_track_element_class_init (GESTrackElementClass * klass)
   object_class->get_property = ges_track_element_get_property;
   object_class->set_property = ges_track_element_set_property;
   object_class->dispose = ges_track_element_dispose;
-  object_class->finalize = ges_track_element_finalize;
+  object_class->constructed = ges_track_element_constructed;
 
 
   /**
@@ -255,7 +280,7 @@ ges_track_element_class_init (GESTrackElementClass * klass)
    * @track_element: a #GESTrackElement
    * @control_binding: the #GstControlBinding that has been added
    *
-   * The control-bunding-added  is emitted each time a control binding
+   * The control-binding-added signal is emitted each time a control binding
    * is added for a child property of @track_element
    */
   ges_track_element_signals[CONTROL_BINDING_ADDED] =
@@ -264,15 +289,15 @@ ges_track_element_class_init (GESTrackElementClass * klass)
       G_TYPE_NONE, 1, GST_TYPE_CONTROL_BINDING);
 
   /**
-   * GESTrackElement::control-binding-added:
+   * GESTrackElement::control-binding-removed:
    * @track_element: a #GESTrackElement
-   * @control_binding: the #GstControlBinding that has been added
+   * @control_binding: the #GstControlBinding that has been removed
    *
-   * The control-bunding-added  is emitted each time a control binding
-   * is added for a child property of @track_element
+   * The control-binding-removed signal is emitted each time a control binding
+   * is removed for a child property of @track_element
    */
   ges_track_element_signals[CONTROL_BINDING_REMOVED] =
-      g_signal_new ("control-binding-reomved", G_TYPE_FROM_CLASS (klass),
+      g_signal_new ("control-binding-removed", G_TYPE_FROM_CLASS (klass),
       G_SIGNAL_RUN_FIRST, 0, NULL, NULL, g_cclosure_marshal_generic,
       G_TYPE_NONE, 1, GST_TYPE_CONTROL_BINDING);
 
@@ -295,11 +320,12 @@ ges_track_element_init (GESTrackElement * self)
       GES_TYPE_TRACK_ELEMENT, GESTrackElementPrivate);
 
   /* Sane default values */
-  priv->pending_start = 0;
-  priv->pending_inpoint = 0;
-  priv->pending_duration = GST_SECOND;
-  priv->pending_priority = MIN_NLE_PRIO;
-  priv->pending_active = TRUE;
+  GES_TIMELINE_ELEMENT_START (self) = 0;
+  GES_TIMELINE_ELEMENT_INPOINT (self) = 0;
+  GES_TIMELINE_ELEMENT_DURATION (self) = GST_SECOND;
+  GES_TIMELINE_ELEMENT_PRIORITY (self) = 0;
+  self->active = TRUE;
+
   priv->bindings_hashtable = g_hash_table_new_full (g_str_hash, g_str_equal,
       g_free, NULL);
 }
@@ -435,13 +461,12 @@ _set_start (GESTimelineElement * element, GstClockTime start)
 {
   GESTrackElement *object = GES_TRACK_ELEMENT (element);
 
-  if (object->priv->nleobject != NULL) {
-    if (G_UNLIKELY (start == _START (object)))
-      return FALSE;
+  g_return_val_if_fail (object->priv->nleobject, FALSE);
 
-    g_object_set (object->priv->nleobject, "start", start, NULL);
-  } else
-    object->priv->pending_start = start;
+  if (G_UNLIKELY (start == _START (object)))
+    return FALSE;
+
+  g_object_set (object->priv->nleobject, "start", start, NULL);
 
   return TRUE;
 }
@@ -451,15 +476,13 @@ _set_inpoint (GESTimelineElement * element, GstClockTime inpoint)
 {
   GESTrackElement *object = GES_TRACK_ELEMENT (element);
 
-  if (object->priv->nleobject != NULL) {
-    if (G_UNLIKELY (inpoint == _INPOINT (object)))
+  g_return_val_if_fail (object->priv->nleobject, FALSE);
 
-      return FALSE;
+  if (G_UNLIKELY (inpoint == _INPOINT (object)))
 
-    g_object_set (object->priv->nleobject, "inpoint", inpoint, NULL);
-  } else
-    object->priv->pending_inpoint = inpoint;
+    return FALSE;
 
+  g_object_set (object->priv->nleobject, "inpoint", inpoint, NULL);
   _update_control_bindings (element, inpoint, GST_CLOCK_TIME_NONE);
 
   return TRUE;
@@ -471,17 +494,16 @@ _set_duration (GESTimelineElement * element, GstClockTime duration)
   GESTrackElement *object = GES_TRACK_ELEMENT (element);
   GESTrackElementPrivate *priv = object->priv;
 
+  g_return_val_if_fail (object->priv->nleobject, FALSE);
+
   if (GST_CLOCK_TIME_IS_VALID (_MAXDURATION (element)) &&
       duration > _INPOINT (object) + _MAXDURATION (element))
     duration = _MAXDURATION (element) - _INPOINT (object);
 
-  if (priv->nleobject != NULL) {
-    if (G_UNLIKELY (duration == _DURATION (object)))
-      return FALSE;
+  if (G_UNLIKELY (duration == _DURATION (object)))
+    return FALSE;
 
-    g_object_set (priv->nleobject, "duration", duration, NULL);
-  } else
-    priv->pending_duration = duration;
+  g_object_set (priv->nleobject, "duration", duration, NULL);
 
   _update_control_bindings (element, ges_timeline_element_get_inpoint (element),
       duration);
@@ -494,21 +516,20 @@ _set_priority (GESTimelineElement * element, guint32 priority)
 {
   GESTrackElement *object = GES_TRACK_ELEMENT (element);
 
+  g_return_val_if_fail (object->priv->nleobject, FALSE);
+
   if (priority < MIN_NLE_PRIO) {
     GST_INFO_OBJECT (element, "Priority (%d) < MIN_NLE_PRIO, setting it to %d",
         priority, MIN_NLE_PRIO);
     priority = MIN_NLE_PRIO;
   }
 
-  GST_DEBUG ("object:%p, priority:%" G_GUINT32_FORMAT, object, priority);
+  GST_DEBUG_OBJECT (object, "priority:%" G_GUINT32_FORMAT, priority);
 
-  if (object->priv->nleobject != NULL) {
-    if (G_UNLIKELY (priority == _PRIORITY (object)))
-      return FALSE;
+  if (G_UNLIKELY (priority == _PRIORITY (object)))
+    return FALSE;
 
-    g_object_set (object->priv->nleobject, "priority", priority, NULL);
-  } else
-    object->priv->pending_priority = priority;
+  g_object_set (object->priv->nleobject, "priority", priority, NULL);
 
   return TRUE;
 }
@@ -533,22 +554,20 @@ gboolean
 ges_track_element_set_active (GESTrackElement * object, gboolean active)
 {
   g_return_val_if_fail (GES_IS_TRACK_ELEMENT (object), FALSE);
+  g_return_val_if_fail (object->priv->nleobject, FALSE);
 
   GST_DEBUG_OBJECT (object, "object:%p, active:%d", object, active);
 
-  if (object->priv->nleobject != NULL) {
-    if (G_UNLIKELY (active == object->active))
-      return FALSE;
+  if (G_UNLIKELY (active == object->active))
+    return FALSE;
 
-    g_object_set (object->priv->nleobject, "active", active, NULL);
+  g_object_set (object->priv->nleobject, "active", active, NULL);
 
-    if (active != object->active) {
-      object->active = active;
-      if (GES_TRACK_ELEMENT_GET_CLASS (object)->active_changed)
-        GES_TRACK_ELEMENT_GET_CLASS (object)->active_changed (object, active);
-    }
-  } else
-    object->priv->pending_active = active;
+  if (active != object->active) {
+    object->active = active;
+    if (GES_TRACK_ELEMENT_GET_CLASS (object)->active_changed)
+      GES_TRACK_ELEMENT_GET_CLASS (object)->active_changed (object, active);
+  }
 
   return TRUE;
 }
@@ -652,76 +671,6 @@ add_failure:
   }
 }
 
-static gboolean
-ensure_nle_object (GESTrackElement * object)
-{
-  GESTrackElementClass *class;
-  GstElement *nleobject;
-  gboolean res = TRUE;
-
-  if (object->priv->nleobject && object->priv->valid)
-    return FALSE;
-
-  /* 1. Create the NleObject */
-  GST_DEBUG ("Creating NleObject");
-
-  class = GES_TRACK_ELEMENT_GET_CLASS (object);
-
-  if (G_UNLIKELY (class->create_gnl_object == NULL)) {
-    GST_ERROR ("No 'create_gnl_object' implementation !");
-    goto done;
-  }
-
-  GST_DEBUG ("Calling virtual method");
-
-  /* 2. Fill in the NleObject */
-  if (object->priv->nleobject == NULL) {
-
-    /* call the create_gnl_object virtual method */
-    nleobject = class->create_gnl_object (object);
-
-    if (G_UNLIKELY (nleobject == NULL)) {
-      GST_ERROR
-          ("'create_gnl_object' implementation returned TRUE but no NleObject is available");
-      goto done;
-    }
-
-    GST_DEBUG_OBJECT (object, "Got a valid NleObject, now filling it in");
-
-    object->priv->nleobject = gst_object_ref (nleobject);
-    g_object_set_qdata (G_OBJECT (nleobject), NLE_OBJECT_TRACK_ELEMENT_QUARK,
-        object);
-
-    /* Set some properties on the NleObject */
-    g_object_set (object->priv->nleobject,
-        "duration", object->priv->pending_duration,
-        "start", object->priv->pending_start,
-        "inpoint", object->priv->pending_inpoint,
-        "priority", object->priv->pending_priority,
-        "active", object->priv->pending_active, NULL);
-
-    /* Pendings values are not pending anymore */
-    GES_TIMELINE_ELEMENT_START (object) = object->priv->pending_start;
-    GES_TIMELINE_ELEMENT_INPOINT (object) = object->priv->pending_inpoint;
-    GES_TIMELINE_ELEMENT_DURATION (object) = object->priv->pending_duration;
-    GES_TIMELINE_ELEMENT_PRIORITY (object) = object->priv->pending_priority;
-    object->active = object->priv->pending_active;
-
-
-    if (object->priv->track != NULL)
-      g_object_set (object->priv->nleobject,
-          "caps", ges_track_get_caps (object->priv->track), NULL);
-
-  }
-
-done:
-  object->priv->valid = res;
-
-  GST_DEBUG ("Returning res:%d", res);
-
-  return res;
-}
-
 /**
  * ges_track_element_add_children_props:
  * @self: The #GESTrackElement to set chidlren props on
@@ -767,7 +716,7 @@ ges_track_element_add_children_props (GESTrackElement * self,
         continue;
       }
 
-      if (pspec->flags & G_PARAM_WRITABLE) {
+      if (pspec->flags) {
         ges_timeline_element_add_child_property (GES_TIMELINE_ELEMENT (self),
             pspec, G_OBJECT (element));
         GST_LOG_OBJECT (self,
@@ -854,46 +803,20 @@ ges_track_element_add_children_props (GESTrackElement * self,
 }
 
 /* INTERNAL USAGE */
-static void
-_free_pending_binding (PendingBinding * pend)
-{
-  g_slice_free (PendingBinding, pend);
-}
-
 gboolean
 ges_track_element_set_track (GESTrackElement * object, GESTrack * track)
 {
   gboolean ret = TRUE;
-  GST_DEBUG ("object:%p, track:%p", object, track);
+
+  g_return_val_if_fail (object->priv->nleobject, FALSE);
+
+  GST_DEBUG_OBJECT (object, "new track: %" GST_PTR_FORMAT, track);
 
   object->priv->track = track;
 
   if (object->priv->track) {
-    /* If we already have a nleobject, we just set its caps properly */
-    if (object->priv->nleobject) {
-      g_object_set (object->priv->nleobject,
-          "caps", ges_track_get_caps (object->priv->track), NULL);
-    } else {
-      ret = ensure_nle_object (object);
-
-      /* if we had pending control bindings, add them and free them */
-      if (ret && object->priv->pending_bindings) {
-        GList *tmp;
-        PendingBinding *pbinding;
-
-        GST_INFO_OBJECT (object, "Asynchronously adding bindings");
-        for (tmp = object->priv->pending_bindings; tmp; tmp = tmp->next) {
-          pbinding = tmp->data;
-          ges_track_element_set_control_source (pbinding->element,
-              pbinding->source, pbinding->propname, pbinding->binding_type);
-          g_free (pbinding->propname);
-          g_free (pbinding->binding_type);
-        }
-        g_list_free_full (object->priv->pending_bindings,
-            (GDestroyNotify) _free_pending_binding);
-        object->priv->pending_bindings = NULL;
-      }
-    }
+    g_object_set (object->priv->nleobject,
+        "caps", ges_track_get_caps (object->priv->track), NULL);
   }
 
   g_object_notify_by_pspec (G_OBJECT (object), properties[PROP_TRACK]);
@@ -930,8 +853,8 @@ _ges_track_element_get_layer_priority (GESTrackElement * element)
  *
  * Get the #GESTrack to which this object belongs.
  *
- * Returns: (transfer none): The #GESTrack to which this object belongs. Can be %NULL if it
- * is not in any track
+ * Returns: (transfer none) (nullable): The #GESTrack to which this object
+ * belongs. Can be %NULL if it is not in any track
  */
 GESTrack *
 ges_track_element_get_track (GESTrackElement * object)
@@ -1007,11 +930,9 @@ gboolean
 ges_track_element_is_active (GESTrackElement * object)
 {
   g_return_val_if_fail (GES_IS_TRACK_ELEMENT (object), FALSE);
+  g_return_val_if_fail (object->priv->nleobject, FALSE);
 
-  if (G_UNLIKELY (object->priv->nleobject == NULL))
-    return object->priv->pending_active;
-  else
-    return object->active;
+  return object->active;
 }
 
 /**
@@ -1046,7 +967,7 @@ ges_track_element_lookup_child (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_property_by_pspec:
+ * ges_track_element_set_child_property_by_pspec: (skip):
  * @object: a #GESTrackElement
  * @pspec: The #GParamSpec that specifies the property you want to set
  * @value: the value
@@ -1068,7 +989,7 @@ ges_track_element_set_child_property_by_pspec (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_property_valist:
+ * ges_track_element_set_child_property_valist: (skip):
  * @object: The #GESTrackElement parent object
  * @first_property_name: The name of the first property to set
  * @var_args: value for the first property, followed optionally by more
@@ -1090,7 +1011,7 @@ ges_track_element_set_child_property_valist (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_properties:
+ * ges_track_element_set_child_properties: (skip):
  * @object: The #GESTrackElement parent object
  * @first_property_name: The name of the first property to set
  * @...: value for the first property, followed optionally by more
@@ -1118,7 +1039,7 @@ ges_track_element_set_child_properties (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_property_valist:
+ * ges_track_element_get_child_property_valist: (skip):
  * @object: The #GESTrackElement parent object
  * @first_property_name: The name of the first property to get
  * @var_args: value for the first property, followed optionally by more
@@ -1162,7 +1083,7 @@ ges_track_element_list_children_properties (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_properties:
+ * ges_track_element_get_child_properties: (skip):
  * @object: The origin #GESTrackElement
  * @first_property_name: The name of the first property to get
  * @...: return location for the first property, followed optionally by more
@@ -1187,7 +1108,7 @@ ges_track_element_get_child_properties (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_property_by_pspec:
+ * ges_track_element_get_child_property_by_pspec: (skip):
  * @object: a #GESTrackElement
  * @pspec: The #GParamSpec that specifies the property you want to get
  * @value: (out): return location for the value
@@ -1205,7 +1126,7 @@ ges_track_element_get_child_property_by_pspec (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_set_child_property:
+ * ges_track_element_set_child_property: (skip):
  * @object: The origin #GESTrackElement
  * @property_name: The name of the property
  * @value: the value
@@ -1229,7 +1150,7 @@ ges_track_element_set_child_property (GESTrackElement * object,
 }
 
 /**
- * ges_track_element_get_child_property:
+ * ges_track_element_get_child_property: (skip):
  * @object: The origin #GESTrackElement
  * @property_name: The name of the property
  * @value: (out): return location for the property value, it will
@@ -1275,11 +1196,12 @@ ges_track_element_copy_properties (GESTimelineElement * element,
   GValue val = { 0 };
   GESTrackElement *copy = GES_TRACK_ELEMENT (elementcopy);
 
-  ensure_nle_object (copy);
   specs =
       ges_track_element_list_children_properties (GES_TRACK_ELEMENT (element),
       &n_specs);
   for (n = 0; n < n_specs; ++n) {
+    if (!(specs[n]->flags & G_PARAM_WRITABLE))
+      continue;
     g_value_init (&val, specs[n]->value_type);
     ges_track_element_get_child_property_by_pspec (GES_TRACK_ELEMENT (element),
         specs[n], &val);
@@ -1477,7 +1399,6 @@ ges_track_element_edit (GESTrackElement * object,
  * ges_track_element_remove_control_binding:
  * @object: the #GESTrackElement on which to set a control binding
  * @property_name: The name of the property to control.
- * @binding_type: The type of binding to create. Only "direct" is available for now.
  *
  * Removes a #GstControlBinding from @object.
  *
@@ -1523,7 +1444,7 @@ ges_track_element_remove_control_binding (GESTrackElement * object,
 /**
  * ges_track_element_set_control_source:
  * @object: the #GESTrackElement on which to set a control binding
- * @source: (element-type GstControlSource): the #GstControlSource to set on the binding.
+ * @source: the #GstControlSource to set on the binding.
  * @property_name: The name of the property to control.
  * @binding_type: The type of binding to create. Only "direct" is available for now.
  *
@@ -1554,19 +1475,6 @@ ges_track_element_set_control_source (GESTrackElement * object,
     return FALSE;
   }
 
-  if (!priv->track) {
-    PendingBinding *pbinding;
-
-    GST_INFO ("Adding this source to the future bindings");
-    pbinding = g_slice_new0 (PendingBinding);
-    pbinding->element = object;
-    pbinding->source = source;
-    pbinding->propname = g_strdup (property_name);
-    pbinding->binding_type = g_strdup (binding_type);
-    priv->pending_bindings = g_list_append (priv->pending_bindings, pbinding);
-    return TRUE;
-  }
-
   if (!ges_track_element_lookup_child (object, property_name, &element, &pspec)) {
     GST_WARNING ("You need to provide a valid and controllable property name");
     return FALSE;
@@ -1617,8 +1525,8 @@ ges_track_element_set_control_source (GESTrackElement * object,
  * Looks up the various controlled properties for that #GESTrackElement,
  * and returns the #GstControlBinding which controls @property_name.
  *
- * Returns: (transfer none): the #GstControlBinding associated with @property_name, or %NULL
- * if that property is not controlled.
+ * Returns: (transfer none) (nullable): the #GstControlBinding associated with
+ * @property_name, or %NULL if that property is not controlled.
  */
 GstControlBinding *
 ges_track_element_get_control_binding (GESTrackElement * object,
