@@ -292,6 +292,7 @@ _ghost_nlecomposition_srcpad (GESTrack * track)
 
   gst_pad_link (pad, capsfilter_sink);
   gst_object_unref (capsfilter_sink);
+  gst_object_unref (pad);
 
   capsfilter_src = gst_element_get_static_pad (priv->capsfilter, "src");
   /* ghost the pad */
@@ -388,6 +389,18 @@ dispose_trackelements_foreach (GESTrackElement * trackelement, GESTrack * track)
   remove_object_internal (track, trackelement);
 }
 
+/* GstElement virtual methods */
+
+static GstStateChangeReturn
+ges_track_change_state (GstElement * element, GstStateChange transition)
+{
+  if (transition == GST_STATE_CHANGE_READY_TO_PAUSED)
+    track_resort_and_fill_gaps (GES_TRACK (element));
+
+  return GST_ELEMENT_CLASS (ges_track_parent_class)->change_state (element,
+      transition);
+}
+
 /* GObject virtual methods */
 static void
 ges_track_get_property (GObject * object, guint property_id,
@@ -454,9 +467,6 @@ ges_track_dispose (GObject * object)
   g_list_free_full (priv->gaps, (GDestroyNotify) free_gap);
   ges_nle_object_commit (track->priv->composition, TRUE);
 
-  if (priv->mixing_operation)
-    gst_object_unref (priv->mixing_operation);
-
   if (priv->composition) {
     gst_element_remove_pad (GST_ELEMENT (track), priv->srcpad);
     gst_bin_remove (GST_BIN (object), priv->composition);
@@ -522,6 +532,7 @@ ges_track_constructed (GObject * object)
     nleobject = gst_element_factory_make ("nleoperation", "mixing-operation");
     if (!gst_bin_add (GST_BIN (nleobject), mixer)) {
       GST_WARNING_OBJECT (self, "Could not add the mixer to our composition");
+      gst_object_unref (mixer);
       gst_object_unref (nleobject);
 
       return;
@@ -531,12 +542,13 @@ ges_track_constructed (GObject * object)
     if (self->priv->mixing) {
       if (!ges_nle_composition_add_object (self->priv->composition, nleobject)) {
         GST_WARNING_OBJECT (self, "Could not add the mixer to our composition");
+        gst_object_unref (nleobject);
 
         return;
       }
     }
 
-    self->priv->mixing_operation = gst_object_ref (nleobject);
+    self->priv->mixing_operation = nleobject;
 
   } else {
     GST_INFO_OBJECT (self, "No way to create a main mixer");
@@ -551,6 +563,8 @@ ges_track_class_init (GESTrackClass * klass)
 
   g_type_class_add_private (klass, sizeof (GESTrackPrivate));
 
+  gstelement_class->change_state = GST_DEBUG_FUNCPTR (ges_track_change_state);
+
   object_class->get_property = ges_track_get_property;
   object_class->set_property = ges_track_set_property;
   object_class->dispose = ges_track_dispose;
@@ -697,7 +711,7 @@ ges_track_init (GESTrack * self)
  * The newly created track will steal a reference to the caps. If you wish to
  * use those caps elsewhere, you will have to take an extra reference.
  *
- * Returns: A new #GESTrack.
+ * Returns: (transfer floating): A new #GESTrack.
  */
 GESTrack *
 ges_track_new (GESTrackType type, GstCaps * caps)
@@ -875,8 +889,6 @@ ges_track_set_mixing (GESTrack * track, gboolean mixing)
   }
 
   if (mixing) {
-    /* increase ref count to hold the object */
-    gst_object_ref (track->priv->mixing_operation);
     if (!ges_nle_composition_add_object (track->priv->composition,
             track->priv->mixing_operation)) {
       GST_WARNING_OBJECT (track, "Could not add the mixer to our composition");
@@ -1047,7 +1059,7 @@ ges_track_get_caps (GESTrack * track)
  *
  * Get the #GESTimeline this track belongs to. Can be %NULL.
  *
- * Returns: The #GESTimeline this track belongs to. Can be %NULL.
+ * Returns: (nullable): The #GESTimeline this track belongs to. Can be %NULL.
  */
 const GESTimeline *
 ges_track_get_timeline (GESTrack * track)
