@@ -344,6 +344,7 @@ static void
 discoverer_discovered_cb (GstDiscoverer * discoverer,
     GstDiscovererInfo * info, GError * err, gpointer user_data)
 {
+  GError *error = NULL;
   const GstTagList *tags;
 
   const gchar *uri = gst_discoverer_info_get_uri (info);
@@ -356,13 +357,20 @@ discoverer_discovered_cb (GstDiscoverer * discoverer,
 
   if (gst_discoverer_info_get_result (info) == GST_DISCOVERER_OK) {
     ges_uri_clip_asset_set_info (mfs, info);
-  } else if (!err) {
-    err = g_error_new (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
-        "Stream %s discovering failed (error code: %d)",
-        uri, gst_discoverer_info_get_result (info));
+  } else {
+    if (err) {
+      error = g_error_copy (err);
+    } else {
+      error = g_error_new (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
+          "Stream %s discovering failed (error code: %d)",
+          uri, gst_discoverer_info_get_result (info));
+    }
   }
 
-  ges_asset_cache_set_loaded (GES_TYPE_URI_CLIP, uri, err);
+  ges_asset_cache_set_loaded (GES_TYPE_URI_CLIP, uri, error);
+
+  if (error)
+    g_error_free (error);
 }
 
 /* API implementation */
@@ -468,7 +476,8 @@ ges_uri_clip_asset_new (const gchar * uri, GCancellable * cancellable,
  * Creates a #GESUriClipAsset for @uri syncronously. You should avoid
  * to use it in application, and rather create #GESUriClipAsset asynchronously
  *
- * Returns: (transfer none): A reference to the requested asset or %NULL if an error happend
+ * Returns: (transfer none) (nullable): A reference to the requested asset or
+ * %NULL if an error happened
  */
 GESUriClipAsset *
 ges_uri_clip_asset_request_sync (const gchar * uri, GError ** error)
@@ -511,12 +520,24 @@ ges_uri_clip_asset_request_sync (const gchar * uri, GError ** error)
     info = gst_discoverer_discover_uri (discoverer, uri, &lerror);
   }
 
+  /* We might get a discoverer info but it might have a non-OK result. We
+   * should consider that an error */
+  if (!lerror && info
+      && gst_discoverer_info_get_result (info) != GST_DISCOVERER_OK) {
+    lerror =
+        g_error_new (GST_RESOURCE_ERROR, GST_RESOURCE_ERROR_FAILED,
+        "Stream %s discovering failed (error code: %d)", uri,
+        gst_discoverer_info_get_result (info));
+  }
+
   ges_asset_cache_put (gst_object_ref (asset), NULL);
   ges_uri_clip_asset_set_info (asset, info);
   ges_asset_cache_set_loaded (GES_TYPE_URI_CLIP, uri, lerror);
 
   if (info == NULL || lerror != NULL) {
     gst_object_unref (asset);
+    if (info)
+      gst_discoverer_info_unref (info);
     if (lerror)
       g_propagate_error (error, lerror);
 
@@ -577,6 +598,7 @@ G_DEFINE_TYPE (GESUriSourceAsset, ges_uri_source_asset,
 static GESExtractable *
 _extract (GESAsset * asset, GError ** error)
 {
+  gchar *uri = NULL;
   GESTrackElement *trackelement;
   GESUriSourceAssetPrivate *priv = GES_URI_SOURCE_ASSET (asset)->priv;
 
@@ -592,24 +614,24 @@ _extract (GESAsset * asset, GError ** error)
     return NULL;
   }
 
+  uri = g_strdup (priv->uri);
+
   if (g_str_has_prefix (priv->uri, GES_MULTI_FILE_URI_PREFIX)) {
-    trackelement =
-        GES_TRACK_ELEMENT (ges_multi_file_source_new (g_strdup (priv->uri)));
+    trackelement = GES_TRACK_ELEMENT (ges_multi_file_source_new (uri));
   } else if (GST_IS_DISCOVERER_VIDEO_INFO (priv->sinfo)
       && gst_discoverer_video_info_is_image ((GstDiscovererVideoInfo *)
           priv->sinfo))
-    trackelement =
-        GES_TRACK_ELEMENT (ges_image_source_new (g_strdup (priv->uri)));
+    trackelement = GES_TRACK_ELEMENT (ges_image_source_new (uri));
   else if (GST_IS_DISCOVERER_VIDEO_INFO (priv->sinfo))
-    trackelement =
-        GES_TRACK_ELEMENT (ges_video_uri_source_new (g_strdup (priv->uri)));
+    trackelement = GES_TRACK_ELEMENT (ges_video_uri_source_new (uri));
   else
-    trackelement =
-        GES_TRACK_ELEMENT (ges_audio_uri_source_new (g_strdup (priv->uri)));
+    trackelement = GES_TRACK_ELEMENT (ges_audio_uri_source_new (uri));
 
   ges_track_element_set_track_type (trackelement,
       ges_track_element_asset_get_track_type (GES_TRACK_ELEMENT_ASSET (asset)));
 
+  g_free (uri);
+
   return GES_EXTRACTABLE (trackelement);
 }
 
