@@ -124,7 +124,6 @@ ges_video_source_create_element (GESTrackElement * trksrc)
   GstElement *positionner, *videoscale, *videorate, *capsfilter, *videoconvert,
       *deinterlace;
   const gchar *props[] = { "alpha", "posx", "posy", "width", "height", NULL };
-  GESTimelineElement *parent;
 
   if (!source_class->create_source)
     return NULL;
@@ -145,9 +144,6 @@ ges_video_source_create_element (GESTrackElement * trksrc)
       gst_element_factory_make ("videoconvert", "track-element-videoconvert");
   videorate = gst_element_factory_make ("videorate", "track-element-videorate");
   deinterlace = gst_element_factory_make ("deinterlace", "deinterlace");
-  if (deinterlace == NULL) {
-    deinterlace = gst_element_factory_make ("avdeinterlace", "deinterlace");
-  }
   capsfilter =
       gst_element_factory_make ("capsfilter", "track-element-capsfilter");
 
@@ -172,14 +168,9 @@ ges_video_source_create_element (GESTrackElement * trksrc)
         capsfilter, NULL);
   }
 
-  parent = ges_timeline_element_get_parent (GES_TIMELINE_ELEMENT (trksrc));
-  if (parent) {
-    self->priv->positionner = GST_FRAME_POSITIONNER (positionner);
-    gst_object_unref (parent);
-  } else {
-    GST_ERROR ("No parent timeline element, SHOULD NOT HAPPEN");
-  }
-
+  self->priv->positionner = GST_FRAME_POSITIONNER (positionner);
+  self->priv->positionner->scale_in_compositor =
+      !GES_VIDEO_SOURCE_GET_CLASS (self)->ABI.abi.disable_scale_in_compositor;
   self->priv->capsfilter = capsfilter;
 
   return topbin;
