@@ -38,8 +38,10 @@ static GstFlowReturn gst_frame_positionner_transform_ip (GstBaseTransform *
     trans, GstBuffer * buf);
 
 static gboolean
-gst_frame_positionner_meta_transform (GstBuffer * dest, GstMeta * meta,
-    GstBuffer * buffer, GQuark type, gpointer data);
+gst_frame_positionner_meta_init (GstMeta * meta, gpointer params,
+    GstBuffer * buffer);
+static gboolean gst_frame_positionner_meta_transform (GstBuffer * dest,
+    GstMeta * meta, GstBuffer * buffer, GQuark type, gpointer data);
 
 enum
 {
@@ -77,21 +79,20 @@ _weak_notify_cb (GstFramePositionner * pos, GObject * old)
 
 static void
 gst_frame_positionner_update_properties (GstFramePositionner * pos,
-    gint old_track_width, gint old_track_height)
+    gboolean track_mixing, gint old_track_width, gint old_track_height)
 {
   GstCaps *caps;
 
   if (pos->capsfilter == NULL)
     return;
 
-  if (pos->track_width && pos->track_height) {
+  if (pos->track_width && pos->track_height &&
+      (!track_mixing || !pos->scale_in_compositor)) {
     caps =
         gst_caps_new_simple ("video/x-raw", "width", G_TYPE_INT,
-        pos->track_width, "height", G_TYPE_INT, pos->track_height,
-        "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1, NULL);
+        pos->track_width, "height", G_TYPE_INT, pos->track_height, NULL);
   } else {
-    caps = gst_caps_new_simple ("video/x-raw",
-        "pixel-aspect-ratio", GST_TYPE_FRACTION, 1, 1, NULL);
+    caps = gst_caps_new_empty_simple ("video/x-raw");
   }
 
   if (pos->fps_n != -1)
@@ -99,9 +100,11 @@ gst_frame_positionner_update_properties (GstFramePositionner * pos,
         pos->fps_d, NULL);
 
   if (old_track_width && pos->width == old_track_width &&
-      old_track_width && pos->width == old_track_width) {
+      old_track_height && pos->height == old_track_height &&
+      ((float) old_track_width / (float) old_track_height) ==
+      ((float) pos->track_width / (float) pos->track_height)) {
 
-    GST_DEBUG_OBJECT (pos, "FOLLOWING track size width old_track: %d -- pos: %d"
+    GST_DEBUG_OBJECT (pos, "Following track size width old_track: %d -- pos: %d"
         " || height, old_track %d -- pos: %d",
         old_track_width, pos->width, old_track_height, pos->height);
 
@@ -109,7 +112,7 @@ gst_frame_positionner_update_properties (GstFramePositionner * pos,
     pos->height = pos->track_height;
   }
 
-  GST_DEBUG_OBJECT (pos, "setting caps : %s", gst_caps_to_string (caps));
+  GST_DEBUG_OBJECT (caps, "setting caps");
 
   g_object_set (pos->capsfilter, "caps", caps, NULL);
 
@@ -117,10 +120,13 @@ gst_frame_positionner_update_properties (GstFramePositionner * pos,
 }
 
 static void
-sync_properties_from_caps (GstFramePositionner * pos, GstCaps * caps)
+sync_properties_from_track (GstFramePositionner * pos, GESTrack * track)
 {
   gint width, height;
   gint old_track_width, old_track_height;
+  GstCaps *caps;
+
+  g_object_get (track, "restriction-caps", &caps, NULL);
 
   width = height = 0;
 
@@ -146,29 +152,19 @@ sync_properties_from_caps (GstFramePositionner * pos, GstCaps * caps)
   GST_DEBUG_OBJECT (pos, "syncing framerate from caps : %d/%d", pos->fps_n,
       pos->fps_d);
 
-  gst_frame_positionner_update_properties (pos, old_track_width,
-      old_track_height);
-}
-
-static void
-sync_properties_with_track (GstFramePositionner * pos, GESTrack * track)
-{
-  GstCaps *caps;
-
-  g_object_get (track, "restriction-caps", &caps, NULL);
-  sync_properties_from_caps (pos, caps);
+  gst_frame_positionner_update_properties (pos, ges_track_get_mixing (track),
+      old_track_width, old_track_height);
 }
 
 static void
 _track_restriction_changed_cb (GESTrack * track, GParamSpec * arg G_GNUC_UNUSED,
     GstFramePositionner * pos)
 {
-  sync_properties_with_track (pos, track);
+  sync_properties_from_track (pos, track);
 }
 
 static void
-_track_changed_cb (GESTrackElement * trksrc, GParamSpec * arg G_GNUC_UNUSED,
-    GstFramePositionner * pos)
+set_track (GstFramePositionner * pos)
 {
   GESTrack *new_track;
 
@@ -179,17 +175,26 @@ _track_changed_cb (GESTrackElement * trksrc, GParamSpec * arg G_GNUC_UNUSED,
         (GWeakNotify) _weak_notify_cb, pos);
   }
 
-  new_track = ges_track_element_get_track (trksrc);
+  new_track = ges_track_element_get_track (pos->track_source);
   if (new_track) {
     pos->current_track = new_track;
     g_object_weak_ref (G_OBJECT (new_track), (GWeakNotify) _weak_notify_cb,
         pos);
     GST_DEBUG_OBJECT (pos, "connecting to track : %p", pos->current_track);
+
     g_signal_connect (pos->current_track, "notify::restriction-caps",
         (GCallback) _track_restriction_changed_cb, pos);
-    sync_properties_with_track (pos, pos->current_track);
-  } else
+    sync_properties_from_track (pos, pos->current_track);
+  } else {
     pos->current_track = NULL;
+  }
+}
+
+static void
+_track_changed_cb (GESTrackElement * trksrc, GParamSpec * arg G_GNUC_UNUSED,
+    GstFramePositionner * pos)
+{
+  set_track (pos);
 }
 
 void
@@ -200,18 +205,9 @@ ges_frame_positionner_set_source_and_filter (GstFramePositionner * pos,
   pos->capsfilter = capsfilter;
   pos->current_track = ges_track_element_get_track (trksrc);
 
-  g_object_add_weak_pointer (G_OBJECT (pos->track_source),
-      ((gpointer *) & pos->track_source));
-  g_object_weak_ref (G_OBJECT (pos->current_track),
-      (GWeakNotify) _weak_notify_cb, pos);
-
-  GST_DEBUG_OBJECT (pos, "connecting to track : %p", pos->current_track);
-
-  g_signal_connect (pos->current_track, "notify::restriction-caps",
-      (GCallback) _track_restriction_changed_cb, pos);
   g_signal_connect (trksrc, "notify::track", (GCallback) _track_changed_cb,
       pos);
-  sync_properties_with_track (pos, pos->current_track);
+  set_track (pos);
 }
 
 static void
@@ -335,6 +331,7 @@ gst_frame_positionner_init (GstFramePositionner * framepositionner)
   framepositionner->capsfilter = NULL;
   framepositionner->track_source = NULL;
   framepositionner->current_track = NULL;
+  framepositionner->scale_in_compositor = TRUE;
 }
 
 void
@@ -342,7 +339,10 @@ gst_frame_positionner_set_property (GObject * object, guint property_id,
     const GValue * value, GParamSpec * pspec)
 {
   GstFramePositionner *framepositionner = GST_FRAME_POSITIONNER (object);
+  gboolean track_mixing = TRUE;
 
+  if (framepositionner->current_track)
+    track_mixing = ges_track_get_mixing (framepositionner->current_track);
 
   GST_OBJECT_LOCK (framepositionner);
   switch (property_id) {
@@ -360,11 +360,13 @@ gst_frame_positionner_set_property (GObject * object, guint property_id,
       break;
     case PROP_WIDTH:
       framepositionner->width = g_value_get_int (value);
-      gst_frame_positionner_update_properties (framepositionner, 0, 0);
+      gst_frame_positionner_update_properties (framepositionner, track_mixing,
+          0, 0);
       break;
     case PROP_HEIGHT:
       framepositionner->height = g_value_get_int (value);
-      gst_frame_positionner_update_properties (framepositionner, 0, 0);
+      gst_frame_positionner_update_properties (framepositionner, track_mixing,
+          0, 0);
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
@@ -431,14 +433,29 @@ gst_frame_positionner_get_info (void)
     const GstMetaInfo *meta =
         gst_meta_register (gst_frame_positionner_meta_api_get_type (),
         "GstFramePositionnerMeta",
-        sizeof (GstFramePositionnerMeta), (GstMetaInitFunction) NULL,
-        (GstMetaFreeFunction) NULL,
-        (GstMetaTransformFunction) gst_frame_positionner_meta_transform);
+        sizeof (GstFramePositionnerMeta), gst_frame_positionner_meta_init,
+        NULL,
+        gst_frame_positionner_meta_transform);
     g_once_init_leave (&meta_info, meta);
   }
   return meta_info;
 }
 
+static gboolean
+gst_frame_positionner_meta_init (GstMeta * meta, gpointer params,
+    GstBuffer * buffer)
+{
+  GstFramePositionnerMeta *smeta;
+
+  smeta = (GstFramePositionnerMeta *) meta;
+
+  smeta->alpha = 0.0;
+  smeta->posx = smeta->posy = smeta->height = smeta->width = 0;
+  smeta->zorder = 0;
+
+  return TRUE;
+}
+
 static gboolean
 gst_frame_positionner_meta_transform (GstBuffer * dest, GstMeta * meta,
     GstBuffer * buffer, GQuark type, gpointer data)
