@@ -170,7 +170,7 @@ struct _NleCompositionPrivate
   gboolean seeking_itself;
   gint real_eos_seqnum;
   gint next_eos_seqnum;
-  gint flush_seqnum;
+  guint32 flush_seqnum;
 
   /* 0 means that we already received the right caps or segment */
   gint seqnum_to_restart_task;
@@ -320,16 +320,28 @@ _remove_actions_for_type (NleComposition * comp, GCallback callback)
   GList *tmp;
 
   ACTIONS_LOCK (comp);
-  for (tmp = comp->priv->actions; tmp; tmp = tmp->next) {
+
+  GST_LOG_OBJECT (comp, "finding action[callback=%s], action count = %d",
+      GST_DEBUG_FUNCPTR_NAME (callback), g_list_length (comp->priv->actions));
+  tmp = g_list_first (comp->priv->actions);
+  while (tmp != NULL) {
     Action *act = tmp->data;
+    GList *removed = NULL;
 
     if (ACTION_CALLBACK (act) == callback) {
+      GST_LOG_OBJECT (comp, "remove action for callback %s",
+          GST_DEBUG_FUNCPTR_NAME (callback));
+      removed = tmp;
       g_closure_unref ((GClosure *) act);
-      comp->priv->actions = g_list_delete_link (comp->priv->actions, tmp);
+      comp->priv->actions = g_list_remove_link (comp->priv->actions, removed);
     }
-  }
-  ACTIONS_UNLOCK (comp);
 
+    tmp = g_list_next (tmp);
+    if (removed)
+      g_list_free (removed);
+  }
+
+  ACTIONS_UNLOCK (comp);
 }
 
 static void
@@ -359,17 +371,25 @@ _execute_actions (NleComposition * comp)
     GValue params[1] = { G_VALUE_INIT };
     GList *lact;
 
+    GST_LOG_OBJECT (comp, "scheduled actions [%d]",
+        g_list_length (priv->actions));
+
     g_value_init (&params[0], G_TYPE_OBJECT);
     g_value_set_object (&params[0], comp);
 
-    lact = priv->actions;
-    priv->actions = priv->actions->next;
+    lact = g_list_first (priv->actions);
+    priv->actions = g_list_remove_link (priv->actions, lact);
     ACTIONS_UNLOCK (comp);
 
     GST_INFO_OBJECT (comp, "Invoking %p:%s",
         lact->data, GST_DEBUG_FUNCPTR_NAME ((ACTION_CALLBACK (lact->data))));
     g_closure_invoke (lact->data, NULL, 1, params, NULL);
     g_value_unset (&params[0]);
+    g_closure_unref (lact->data);
+    g_list_free (lact);
+
+    GST_LOG_OBJECT (comp, "remaining actions [%d]",
+        g_list_length (priv->actions));
   } else {
     ACTIONS_UNLOCK (comp);
   }
@@ -718,7 +738,6 @@ _add_object_func (NleComposition * comp, ChildIOData * childio)
     return;
   }
 
-
   g_hash_table_add (priv->pending_io, object);
 }
 
@@ -737,8 +756,10 @@ _add_add_object_action (NleComposition * comp, NleObject * object)
 }
 
 static void
-_free_action (Action * action, gpointer udata)
+_free_action (gpointer udata, Action * action)
 {
+  GST_LOG ("freeing %p action for %s", action,
+      GST_DEBUG_FUNCPTR_NAME (ACTION_CALLBACK (action)));
   if (ACTION_CALLBACK (action) == _seek_pipeline_func) {
     SeekData *seekd = (SeekData *) udata;
 
@@ -778,6 +799,9 @@ _add_action (NleComposition * comp, GCallback func,
   else
     priv->actions = g_list_append (priv->actions, action);
 
+  GST_LOG_OBJECT (comp, "the number of remaining actions: %d",
+      g_list_length (priv->actions));
+
   SIGNAL_NEW_ACTION (comp);
   ACTIONS_UNLOCK (comp);
 }
@@ -964,10 +988,29 @@ nle_composition_init (NleComposition * comp)
       GST_DEBUG_FUNCPTR (nle_composition_event_handler));
 }
 
+static void
+_remove_each_nleobj (gpointer data, gpointer udata)
+{
+  NleComposition *comp = NLE_COMPOSITION (udata);
+  NleObject *nleobj = NLE_OBJECT (data);
+
+  _nle_composition_remove_object (NLE_COMPOSITION (comp), NLE_OBJECT (nleobj));
+}
+
+static void
+_remove_each_action (gpointer data)
+{
+  Action *action = (Action *) (data);
+
+  GST_LOG ("remove action %p for %s", action,
+      GST_DEBUG_FUNCPTR_NAME (ACTION_CALLBACK (action)));
+  g_closure_invalidate ((GClosure *) action);
+  g_closure_unref ((GClosure *) action);
+}
+
 static void
 nle_composition_dispose (GObject * object)
 {
-  GList *iter;
   NleComposition *comp = NLE_COMPOSITION (object);
   NleCompositionPrivate *priv = comp->priv;
 
@@ -976,30 +1019,16 @@ nle_composition_dispose (GObject * object)
 
   priv->dispose_has_run = TRUE;
 
-  iter = priv->objects_start;
-  while (iter) {
-    GList *next = iter->next;
+  g_list_foreach (priv->objects_start, _remove_each_nleobj, comp);
+  g_list_free (priv->objects_start);
 
-    _nle_composition_remove_object (comp, iter->data);
-    iter = next;
-  }
+  g_list_foreach (priv->expandables, _remove_each_nleobj, comp);
+  g_list_free (priv->expandables);
 
+  g_list_foreach (priv->objects_stop, _remove_each_nleobj, comp);
   g_list_free (priv->objects_stop);
 
-  if (priv->expandables) {
-    GList *iter;
-
-    iter = priv->expandables;
-
-    while (iter) {
-      GList *next = iter->next;
-
-      _nle_composition_remove_object (comp, iter->data);
-      iter = next;
-    }
-
-    priv->expandables = NULL;
-  }
+  g_list_free_full (priv->actions, (GDestroyNotify) _remove_each_action);
 
   nle_composition_reset_target_pad (comp);
 
@@ -1520,13 +1549,23 @@ nle_composition_event_handler (GstPad * ghostpad, GstObject * parent,
   switch (GST_EVENT_TYPE (event)) {
     case GST_EVENT_SEEK:
     {
-      if (!priv->seeking_itself) {
+      /* Queue up a seek action if this seek event does not come from
+       * ourselves. Due to a possible race condition around the
+       * seeking_itself flag, we also check if the seek comes from
+       * our task thread. The seeking_itself flag only works as an
+       * optimization */
+      GST_OBJECT_LOCK (comp);
+      if (!priv->seeking_itself || (comp->task
+              && gst_task_get_state (comp->task) != GST_TASK_STOPPED
+              && g_thread_self () != comp->task->thread)) {
+        GST_OBJECT_UNLOCK (comp);
         _add_seek_action (comp, event);
         event = NULL;
         GST_FIXME_OBJECT (comp, "HANDLE seeking errors!");
 
         return TRUE;
       }
+      GST_OBJECT_UNLOCK (comp);
       break;
     }
     case GST_EVENT_QOS:
@@ -2501,15 +2540,19 @@ _relink_children_recursively (NleComposition * comp,
     _relink_single_node (comp, child, toplevel_seek);
 
   if (G_UNLIKELY (nbchildren < oper->num_sinks))
-    GST_ERROR_OBJECT (comp,
-        "%" GST_PTR_FORMAT
-        " Not enough sinkpads to link all objects to the operation ! "
-        "%d / %d", oper, oper->num_sinks, nbchildren);
+    GST_ELEMENT_ERROR (comp, STREAM, FAILED,
+        ("The NleComposition structure is not valid"),
+        ("%" GST_PTR_FORMAT
+            " Not enough sinkpads to link all objects to the operation ! "
+            "%d / %d, current toplevel seek %" GST_PTR_FORMAT,
+            oper, oper->num_sinks, nbchildren, toplevel_seek));
 
   if (G_UNLIKELY (nbchildren == 0)) {
-    GST_ERROR_OBJECT (comp,
-        "Operation %" GST_PTR_FORMAT
-        " has no child objects to be connected to !!!", oper);
+    GST_ELEMENT_ERROR (comp, STREAM, FAILED,
+        ("The NleComposition structure is not valid"),
+        ("Operation %" GST_PTR_FORMAT
+            " has no child objects to be connected to"
+            "current toplevel seek: %" GST_PTR_FORMAT, oper, toplevel_seek));
   }
   /* Make sure we have enough sinkpads */
 }
@@ -2529,6 +2572,7 @@ _relink_single_node (NleComposition * comp, GNode * node,
 {
   NleObject *newobj;
   NleObject *newparent;
+  GNode *node_it;
   GstPad *srcpad = NULL, *sinkpad = NULL;
   GstEvent *translated_seek;
 
@@ -2541,6 +2585,12 @@ _relink_single_node (NleComposition * comp, GNode * node,
   GST_DEBUG_OBJECT (comp, "newobj:%s",
       GST_ELEMENT_NAME ((GstElement *) newobj));
 
+  newobj->recursive_media_duration_factor = 1.0f;
+  for (node_it = node; node_it != NULL; node_it = node_it->parent) {
+    NleObject *object = (NleObject *) node_it->data;
+    newobj->recursive_media_duration_factor *= object->media_duration_factor;
+  }
+
   srcpad = NLE_OBJECT_SRC (newobj);
 
   gst_bin_add (GST_BIN (comp->priv->current_bin), GST_ELEMENT (newobj));
@@ -3014,7 +3064,7 @@ nle_composition_remove_object (GstBin * bin, GstElement * element)
   NleComposition *comp = (NleComposition *) bin;
 
   if (element == comp->priv->current_bin) {
-    GST_INFO_OBJECT (comp, "Adding internal bin");
+    GST_INFO_OBJECT (comp, "Removing internal bin");
     return GST_BIN_CLASS (parent_class)->remove_element (bin, element);
   }
 
